//--------------------------------------------
// EARTH palette color definitions
//--------------------------------------------
// Copyright (C) 1993, Nicholas Centanni
//--------------------------------------------------
#include <stdio.h>
#include <graphics.h>
#include <op_color.h>
#include <conio.h>
#include "nb_color.h"

int fade_cancel;

// Define the EARTH color palette
palettetype earth =
{
 { 16 },
 {
   EARTH_0, EARTH_1, EARTH_2, EARTH_3,
   EARTH_4, EARTH_5, EARTH_6, EARTH_7,
   EARTH_8, EARTH_9, EARTH_10, EARTH_11,
	EARTH_12, EARTH_13, EARTH_14, EARTH_15
 }
};

rgb_color earth_rgb[16] =
{
  {0, 0, 0},        // {0, 0, 0},        EARTH_BLACK
  {21, 21, 21},     // {85, 85, 85},     EARTH_DK_GRAY
  {42, 42, 42},     // {170, 170, 170},  EARTH_LT_GRAY
  {0, 21, 0},       // {0, 85, 0},       EARTH_DK_GREEN
  {0, 42, 0},       // {0, 170, 0},      EARTH_MED_GREEN
  {21, 63, 21},     // {85, 255, 85},    EARTH_LT_GREEN
  {0, 0, 42},       // {0, 0, 170},      EARTH_DK_BLUE
  {0, 0, 63},       // {0, 0, 255},      EARTH_MED_BLUE
  {0, 42, 63},      // {0, 170, 255},    EARTH_LT_BLUE
  {42, 21, 0},      // {170, 85, 0},     EARTH_BROWN
  {42, 0, 0},       // {170, 0, 0},      EARTH_RED
  {0, 21, 21},      // {0, 85, 85},      EARTH_DK_CYAN
  {0, 42, 42},      // {0, 170, 170},    EARTH_MED_CYAN
  {21, 63, 63},     // {85, 255, 255},   EARTH_LT_CYAN
  {63, 63, 21},     // {255, 255, 85},   EARTH_YELLOW
  {63, 63, 63}      // {255, 255, 255}   EARTH_WHITE
};

//---------------------------------------------------------------------------
// Fades out 3 colors at once
//---------------------------------------------------------------------------
void rgb_fade_out(rgb_color *rgb, int steps, int c1, int c2, int c3)
{
  struct palettetype pal;
  rgb_table table[16];
  rgb_table incs[16];
  getpalette(&pal);
  int i;

  // Abort if a fade-cancel command has already been issued
  if (fade_cancel)
    return;

  // Copy the integer RGB values to floating point
  for (i = 0; i < 16; i++)
  {
    table[i].r = rgb[i].red;
    table[i].g = rgb[i].green;
    table[i].b = rgb[i].blue;
    incs[i].r = table[i].r / steps;
    incs[i].g = table[i].g / steps;
    incs[i].b = table[i].b / steps;
  }

  // Fade the values
  for (int s = 0; s < steps; s++)
  {
    for (i = 1; i < 16; i++)
    {
      if (i == c1 || i == c2 || i == c3)
      {
        table[i].r -= incs[i].r;
        table[i].g -= incs[i].g;
        table[i].b -= incs[i].b;
        setrgbpalette(pal.colors[i], table[i].r, table[i].g, table[i].b);
        if (kbhit()) 
        {
          fade_cancel = 1;
          break;
		  }
      }
    }
  }
}

//---------------------------------------------------------------------------
// Fades in 3 colors at once
//---------------------------------------------------------------------------
void rgb_fade_in(rgb_color *rgb, int steps, int c1, int c2, int c3)
{
  struct palettetype pal;
  rgb_table table[16];
  rgb_table incs[16];
  getpalette(&pal);
  int i;

  // Abort if a fade-cancel command has already been issued
  if (fade_cancel)
    return;

  // Copy the integer RGB values to floating point
  for (i = 0; i < 16; i++)
  {
    table[i].r = rgb[i].red;
    table[i].g = rgb[i].green;
	 table[i].b = rgb[i].blue;
    incs[i].r = (double)rgb[i].red / steps;
    incs[i].g = (double)rgb[i].green / steps;
    incs[i].b = (double)rgb[i].blue / steps;
  }

  // Zero out the colors
  for (i = 0; i < 16; i++)
  {
    table[i].r = 0;
    table[i].g = 0;
    table[i].b = 0;
  }

  // Fade the values
  for (int s = 0; s < steps; s++)
  {
    for (i = 1; i < 16; i++)
    {
      if (i == c1 || i == c2 || i == c3)
      {
        table[i].r += incs[i].r;
        table[i].g += incs[i].g;
        table[i].b += incs[i].b;
        setrgbpalette(pal.colors[i], table[i].r, table[i].g, table[i].b);
        if (kbhit()) 
		  {
          fade_cancel = 1;
          break;
        }
      }
    }
  }
}

#ifdef RGB_TO_HSL
//-------------------------------------------------------------------------
// Convert an RGB color to HSL
//-------------------------------------------------------------------------
void rgb_to_hsl(double r, double g, double b, 
                double *h, double *s, double *l)
{
  double v, m, vm, r2, g2, b2;

  // Init
  v = max(r, g);
  v = max(v, b);
  m = min(r, g);
  m = min(m, b);

  // Start with zero h s l values
  *h = *s = *l = 0.0;

  // Calculate LIGHTNESS
  if ((*l = (m + v) / 2.0) <= 0.0)
    return;

  // Calculate SATURATION
  if ((*s = vm = v - m) > 0.0)
    *s /= (*l <= 0.5) ? (v + m) : (2.0 - v - m);
  else
	 return;

  // Calculate HUE
  r2 = (v - r) / vm;
  g2 = (v - g) / vm;
  b2 = (v - b) / vm;
  if ( r == v )
    *h = (g == m ? 5.0 + b2 : 1.0 - g2);
  else
  {
    if ( g == v )
      *h = (b == m ? 1.0 + r2 : 3.0 - b2);
    else
		*h = (r == m ? 3.0 + g2 : 5.0 - r2);
  }
  *h /= 6;
}
#endif

//-------------------------------------------------------------------------
// Convert an HSL color to RGB
//-------------------------------------------------------------------------
void hsl_to_rgb(double h, double s, double l, 
                double *r, double *g, double *b)
{
  double v;

  v = (l <= 0.5) ? (l * (1.0 + s)) : (l + s - l * s);

  if (v <= 0)
    *r = *g = *b = 0.0;
  else
  {
    double m, sv;
    int sextant;
    double fract, vsf, mid1, mid2;
  
    m = l + l - v;
    sv = (v - m) / v;
    h *= 6.0;
    sextant = h;
    fract = h - sextant;
    vsf = v * sv * fract;
    mid1 = m + vsf;
    mid2 = v - vsf;
    switch (sextant)
    {
      case 0:
        *r = v; *g = mid1; *b = m;
        break;
      case 1:
        *r = mid2; *g = v; *b = m;
        break;
      case 2:
        *r = m; *g = v; *b = mid1;
        break;
      case 3:
        *r = m; *g = mid2; *b = v;
        break;
      case 4:
        *r = mid1; *g = m; *b = v;
        break;
      case 5:
        *r = v; *g = m; *b = mid2;
        break;
    }
  }
}
