//--------------------------------------------------
// Header for BIRD object
//--------------------------------------------------
// $Revision:   1.20  $
// $Date:   22 Apr 1994 13:25:18  $
//--------------------------------------------------
#ifndef __OB_BIRD_H
#define __OB_BIRD_H     
#ifndef __NB_VER_H
#include "nb_ver.h"
#endif

#define TOT_VERTICES     21
#define TOT_FACES        19
#define FLAP_STEPS        9   // Must be an odd number

#define GLIDE_ODDS       15
#define GLIDE_WAIT       60
#define SPEED_INC      0.25
#define BIRD_SIZE      0.80

//---------------------------------------------
// Describe each bird vertex
//---------------------------------------------
struct b_vertex
{
  l_point world;
  i_screen pix[2];
  // long distance;
};

//---------------------------------------------
// Describe each bird polygon
//---------------------------------------------
struct b_poly
{
  int v_index[3];
  // long distance;
  int top_shade;
  int under_shade;
};

//------------------------------------------
// BIRD object
//------------------------------------------
class Bird
{
 private:
  b_poly poly[TOT_FACES];         // Body polygons
  b_vertex vert[TOT_VERTICES];    // Body vertices
  l_point pos[2][3][FLAP_STEPS];  // Record of all possible wing positions
  s_point b_box[2];               // Bounding box for fast erase process
  s_point st_box[2];              // Initializer for the bounding box
  int flap_direction;             // Animation data
  int flap_cnt;                   // .
  int flap_mid;                   // .
  int flap_hold;                  // .
  long near_flap_inc;             // .
  long mid_flap_inc;              // .
  long tip_flap_inc;              // .
  int save_direction;             // .
  int save_chg, save_cnt;         // .
  int wait_cnt;                   // .
  double shift_inc;               // .

  //                    
  //   |Pos Z                   TOP        
  //   |                2                
  //   v    18---16--15 0 3---9---11        
  //         |   |    | | |   |    |        
  //    20---19---17--14 6 4--10---12---13    
  //                    / \
  //  -->             /     \              
  //  Pos X          7---5---8             
  //
  long shoulder_width;    // x_dist: 15 -> 3
  long tail_width;        // x_dist: 7 -> 8
  long wing_span_1;       // x_dist: 0 -> 10
  long wing_span_2;       // x_dist: 0 -> 12
  long wing_span_3;       // x_dist: 0 -> 13
  long wing_width_f1;     // z_dist: 0 -> 9  
  long wing_width_f2;     // z_dist: 0 -> 11
  long wing_width_b1;     // z_dist: 0 -> 4 
  long wing_width_b2;     // z_dist: 0 -> 10
  long wing_width_b3;     // z_dist: 0 -> 12
  long wing_width_b4;     // z_dist: 0 -> 13
  long head_length;       // z_dist: 0 -> 2
  long body_length;       // z_dist: 0 -> 5
  long tail_start;        // z_dist: 0 -> 6
  long tail_extra;        // z_dist: 7 -> 5
  //
  //                          FRONT
  //
  //  ^ Pos Y        8-------7
  //  |               \  0  /              
  //  |  13---11--9----3-2-15-16---18---20
  //          12  10   4 1 14 17   19  
  //                    
  // cy = bird->center.y
  //
  long head_height;       // y_dist: cy -> 0
  long beak_height;       // y_dist: cy -> 2
  long throat_height;     // y_dist: cy -> 1
  long tail_height;       // y_dist: cy -> 7

  // Private member functions
  void translate_vertices(void);
  void bounding_box(void);
  void draw_faces(void);
  void shift_body(int direction);
  // void sort_faces(void);
  void drift(int distance);
  void define(void);

 public:
  double speed_chg;
  long half_distance;
  l_point center;                 // Center point
  long curr_z;

  Bird(long hx, long hy, long hz);
  void set_center(long x, long y, long z)
    { center.x = x; center.y = y; center.z = z; }
  void elevation(int direction, long increment);
  void scale_settings(void);
  void flap_wings(void);
  void set_wings(void);
  void change_position(double inc);
  void draw(void);
  void erase(void);
};
#endif
