//----------------------------------------------
// GROUND member functions
//----------------------------------------------
#include <conio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <graphics.h>
#include <opus.h>
#include <op_graph.h>
#include <op_color.h>
#include <op_view.h>
#include "nb_app.h"

//-------------------------------------------------------------------------
// Ground class constructor
//-------------------------------------------------------------------------
Ground::Ground()
{
  max_points = MAX_POINTS;
  color_cnt = COLOR_CHG;
  color_toggle = 0;
  color_index = prev_color_index = COLOR_MARK;

  // Derive an initial color
  color.hue = (double)random(100) / 100;
  color.saturation = 0.80 + (double)random(20) / 100;
  color.lightness = 0.8;
  set_color();

  // Build the initial point array
  start_points();
}

//-------------------------------------------------------------------------
// Ground class destructor
//-------------------------------------------------------------------------
Ground::~Ground()
{
  delete ground_points;
}

//-------------------------------------------------------------------------
// Initialize the full array of ground points
//-------------------------------------------------------------------------
void Ground::start_points()
{
  // Allocate the points array
  while ((ground_points = new ground_point[max_points]) == NULLP)
  {
    if (max_points > POINT_BLOCK)
      max_points -= POINT_BLOCK;
    else
      bail("Mem:ground");
  }

  // Initialize the starting position of the ground points
  memset(ground_points, 0, sizeof(ground_point) * max_points);
  point_count = POINT_BLOCK;
  for (int i = 0; i < point_count; i++)
    start_point(i);
}

//-------------------------------------------------------------------------
// Add a new block of points
//-------------------------------------------------------------------------
void Ground::add_points()
{
  if (point_count + POINT_BLOCK <= max_points)
  {
    for (int i = point_count; i < (point_count + POINT_BLOCK); i++)
      start_point(i);
    point_count += POINT_BLOCK;
  }
}

//-------------------------------------------------------------------------
// Add a new ground detail point to the array
//-------------------------------------------------------------------------
void Ground::start_point(int index)
{
  ground_point *p = &ground_points[index];

  if (--color_cnt == 0)
  {
    prev_color_index = color_index;
    color_toggle = !color_toggle;
    color_index = color_toggle + COLOR_MARK;
    color_cnt = COLOR_CHG;
    color_fade = FADE_CHG;

    // Come up with a new color
    color.hue = (double)random(100) / 100;
    color.saturation = 0.80 + (double)random(20) / 100;
    color.lightness = 0.8;
    set_color();
  }
  if (color_fade)
  {
    int chance = random(2);
    int index = chance ? color_index : prev_color_index;
    color_fade--;
    p->color = index;
  }
  else
    p->color = color_index;

  p->world.x = -X_HORIZON + ((long)random(X_HORIZON) * 2);
  p->world.y = -Z_HORIZON + ((long)random(Z_HORIZON) * 2);
}

//-------------------------------------------------------------------------
// Restart a ground detail point in the array
//-------------------------------------------------------------------------
void Ground::restart_point(int index)
{
  ground_point *p = &ground_points[index];

  if (--color_cnt == 0)
  {
    prev_color_index = color_index;
    color_toggle = !color_toggle;
    color_index = color_toggle + COLOR_MARK;
    color_cnt = COLOR_CHG;
    color_fade = FADE_CHG;

    // Come up with a new color
    color.hue = (double)random(100) / 100;
    color.saturation = 0.80 + (double)random(20) / 100;
    color.lightness = 0.8;
    set_color();
  }
  if (color_fade)
  {
    int chance = random(2);
    int index = chance ? color_index : prev_color_index;
    color_fade--;
    p->color = index;
  }
  else
    p->color = color_index;

  p->world.x = -X_HORIZON + ((long)random(X_HORIZON) * 2);
  p->world.y = -Z_HORIZON;
}

//-------------------------------------------------------------------------
// Move all ground detail points DISTANCE units to the SOUTH
//-------------------------------------------------------------------------
void Ground::move(int distance)
{
  for (int i = 0; i < point_count; i++)
    if ((ground_points[i].world.y += distance) > Z_HORIZON)
      restart_point(i);
}

//-------------------------------------------------------------------------
// Display the ground detail
//-------------------------------------------------------------------------
void Ground::draw()
{
  int page = graphics->active_page;

  for (int i = 0; i < point_count; i++)
  {
    viewer->translate(&ground_points[i].world, &ground_points[i].pix[page]);
    plot_1(ground_points[i].pix[page].x, ground_points[i].pix[page].y, 
           ground_points[i].color);
  }
}

//-------------------------------------------------------------------------
// Erase the ground detail
//-------------------------------------------------------------------------
void Ground::erase()
{
  int page = graphics->active_page;

  for (int i = 0; i < point_count; i++)
    plot_1(ground_points[i].pix[page].x, ground_points[i].pix[page].y, 
           EARTH_BLACK);
}

//-------------------------------------------------------------------------
// Change the current font HSL color to a real RGB value for YELLOW
//-------------------------------------------------------------------------
void Ground::set_color()
{
  double red, green, blue;
  int r, g, b;

  // Convert from HSL to RGB
  hsl_to_rgb(color.hue, color.saturation, color.lightness, 
             &red, &green, &blue);
  r = (int)(red * 63);
  g = (int)(green * 63);
  b = (int)(blue * 63);

  // Assign this custom color to palette entry: YELLOW
  struct palettetype pal;
  getpalette(&pal);
  setrgbpalette(pal.colors[color_index], r, g, b);
}

